mod table_directory;
mod character_map;
mod maximum_profile;
mod font_header;
mod search;

use std::{fs::File, io::{BufReader, Read}};

use bincode::Options;
use character_map::CharacterMap;
use font_header::FontHeader;
use log::{debug, info};
use maximum_profile::MaximumProfile;
use table_directory::TableDirectory;

fn deserialize<R: Read, T: serde::de::DeserializeOwned>(reader: R) -> bincode::Result<T> {
	bincode::options().with_big_endian().with_fixint_encoding().deserialize_from::<R, T>(reader)
}

#[derive(Debug)]
pub struct Font {
	table_directory: TableDirectory,
	font_header: FontHeader,
	character_map: CharacterMap,
	maximum_profile: MaximumProfile
}
	
impl Font {
	pub fn new(file: File) -> bincode::Result<Self> {
		let mut buf_reader = BufReader::new(file.try_clone()?);
		let table_directory = TableDirectory::new(&mut buf_reader)?;

		let font_header = table_directory.get_table(&file)?;
		debug!("{font_header:#?}");

		let character_map = table_directory.get_table(&file)?;
		debug!("{character_map:#?}");

		let maximum_profile = table_directory.get_table(&file)?;
		debug!("{maximum_profile:#?}");

		Ok(Font {
			table_directory,
			font_header,
			character_map,
			maximum_profile
		})
	}

	pub fn get_data_for(&mut self, glyph: char) -> Result<(), &'static str> {
		let Some(glyph_id) = self.character_map.get_char(glyph) else {
			return Err("Failed to get glyph id for char");
		};

		info!("glyph id for char '{glyph}' is {glyph_id}");

		Ok(())
	}
}

